
; NAME :    
;           calcSingScatPartOfReflectancenew
; PURPOSE :
;           Calculate Single Scattering part of MODIS C6 reflectance for a particular
;           sun sattelite geometry
; CALLING :
;           ;;;Lambertian surface
;           calcSingScatPartOfReflectancenew, PF, MSR, sol, view, azm, SSR, PHASE = 'LIQUID'
;           calcSingScatPartOfReflectancenew, PF, MSR, sol, view, azm, SSR, PHASE = 'ICE'
;           sol, view, azm can be scalars or vectors of the same length
;           ;;; Ocean Surface
;           calcSingScatPartOfReflectancenew, PF, MSR, sol, view, azm, SSR, PHASE = 'LIQUID',/OCEAN    
;           calcSingScatPartOfReflectancenew, PF, MSR, sol, view, azm, SSR, PHASE = 'LIQUID',/OCEAN
;           sol, view, azm can be scalars or vectors of the same length
; INPUTS  :
;           PF  = Structure containing phasefunction data
;           MSR = Structure containing single scattering properties and LUT RE & tau values
;           sol    = Solar Angle in degrees
;           view  = View Angle in degrees
;           azm    = Azimuth Angle in degrees
;           PHASE     = Liquid or Ice phase
; OUTPUT  :
;           SSR = Array containing single scattering part
; KEYWORDS :
;           OCEAN = if set, calculate SS reflectance for cox-munk LUTs
;                   if not SS reflectance for lambertian LUTs
; EXTERNAL CALLS:
; function scatAngleDeg (compute scattering angle)
; getAllPhaseFunctionValues (get the phasefunction data from PF LUT)
;           
; HISTORY:
; Written: Nandana Amarasinghe, SSAI @ NASA/GSFC, 2017.
;           
;            
        
pro calcSingScatPartOfReflectancenew,pfStruct,sscStruct, solang,viewang, azmang, ssRefl, PHASE=phase, OCEAN=ocean

  exptol = 12.0
  scatAngle = scatAngleDeg(solang, viewang, azmang)
  RAD_PER_DEG = !PI/180.0
  UMU  = COS(viewang * RAD_PER_DEG)
  UMU0 = COS(solang * RAD_PER_DEG)
  MUPLUSMU0 = UMU + UMU0
  ratio = MUPLUSMU0/(UMU * UMU0)
  
  getAllPhaseFunctionValues, pfStruct, sscStruct.PFNCONST,scatAngle, interpPF, PHASE=phase
 
  tauVals = sscStruct.COT
  RE      = sscStruct.RE
  dfitF   = sscStruct.TRUNCF
  Qe      = sscStruct.EXTCOEFF
  ssAlbedo= sscStruct.SSALB
  
  nscat = n_elements(scatAngle)
  nRE = n_elements(RE)
  ntau = n_elements(tauVals)
  nwl = (size(Qe,/DIMENSIONS))[0]
  fQe = fltarr(nwl,nRE) + 1.0
  for iwl = 1, nwl-1 do begin
    fQe[iwl, *] = Qe[iwl, *]/Qe[0,*]
  end
  
  fOmega = dfitF * ssAlbedo
  
  fPhase = reform(reform(interpPF,nscat,nRE*nwl)#diag_matrix(reform(1.0/ (1.0 - fOmega),nRE*nwl)),nscat,nwl,nRE)
  
  temp = fltarr(nscat,ntau, nwl, nRE)
  nREfromPhase = (size(fPhase,/DIMENSIONS))[2]
  if(nRE ne nREfromPhase)THEN bEGIN
    print,'Please check the to see LUT matches with the correct phase called (ICE or LIQUID)'
    stop
  end

  EXP1 = fltarr(nscat,nwl,nRE)
  IF KEYWORD_SET(ocean) THEN BEGIN
  
    aeroOmega = sscStruct.AEROSSA
    aeroTau   = sscStruct.AEROCOT
    aerog     = sscStruct.AEROG
    RLtau     = sscStruct.RAYCOT
    cosScatAng = COS(scatAngle * !PI/180.0)
    gsquare = aerog*aerog
    RLphase = 3.0 * (1 + (cosScatAng)^2)/4.0
    
    aeroPF = fltarr(nwl,nscat)
    aeroPF= (1.0 - rebin(gsquare,nwl,nscat))/(sqrt((1 + rebin(gsquare,nwl,nscat) - 2.0 * aerog # cosScatAng)^3))
    aeroPF[nwl-2:nwl-1,*] = 0.0
    aeroPhase = transpose(reform(aeroPF))  
    aeroPF=0.0 
    
  
    temp[*,*,*,*] = 0.0
    for itau = 0, ntau-1 do begin

      tauLR =  tauVals[itau] * fQe *(1.0 - fOmega)
;      EXP1=tauLR  ; just initialize here to get the same dimensions, overwrite

      EXP0 = 1.0
      
      temparg = reform(ratio#(reform(tauLR,nwl*nRE)),nscat,nwl,nRE)
      temparg1 = temparg
      temparg = (temparg lt exptol)*temparg
      
      EXP1= (temparg1 lt exptol)*EXP(-temparg*1.d0)
      
      fmultss = reform(reform(fPhase,nscat,nRE*nwl)#diag_matrix(reform(ssAlbedo,nRE*nwl)),nscat,nwl,nRE)
      temp[*,itau,*,*] = temporary(temp[*,itau,*,*]) + (fmultss[*,*,*]) * (EXP0 - EXP1)
      fmultss=0.0
    
      EXP0 = EXP1
      for iwl = 0, nwl-3 do begin
        tauLR[iwl,*] =  tauLR[iwl,*] + RLTau[iwl]
        for ire = 0, nre-1 do begin
          temparg = tauLR[iwl,ire]*ratio
          temparg1 = temparg
          temparg = (temparg lt exptol)*temparg        
          EXP1[*,iwl,ire] = (temparg1 lt exptol) * EXP(-temparg*1.d0)
        end 
        
        temp[*,itau,iwl,*] = temp[*,itau,iwl,*] +  $
                                  diag_matrix(RLphase)#reform((EXP0[*,iwl,*] - EXP1[*,iwl,*]))
      
        EXP0[*,iwl,*] = EXP1[*,iwl,*]
      
        tempOmega = ((aeroTau[iwl] -0.1) * 1.0 + 0.1 * aeroOmega[iwl])/aeroTau[iwl]
        tauLR[iwl,*] = (tauLR[iwl,*]  + aeroTau[iwl]) 
        
        for ire = 0, nre-1 do begin
          temparg = tauLR[iwl,ire]*ratio
          temparg1 = temparg
          temparg = (temparg lt exptol)*temparg          
          EXP1[*,iwl,ire] = (temparg1 lt exptol) * EXP(-temparg*1.d0)
          
        end 
        
        ;need to take care of mixin rayleigh molecules with aerosol 
        ;(that's what is done in DISORT)
        tempPhase = ((aeroTau[iwl] -0.1) * RLphase[*] * 1.0 + $ 
                     0.1 * aeroOmega[iwl] * aeroPhase[*,iwl])/ $
                              ( (aeroTau[iwl] -0.1)*1.0 + 0.1 * aeroOmega[iwl] )
      
        ;coming in aeroTau is the (rayleightau + 0.1) from DISORT run
        temp[*,itau,iwl,*] = temp[*,itau,iwl,*] + tempOmega* $
                                              diag_matrix(tempPhase)#reform(EXP0[*,iwl,*] - EXP1[*,iwl,*])
        
        
                                                      
      end
   end

  ENDIF ELSE BEGIN
  

    for itau = 0, ntau-1 do begin
      tauC = (tauVals[itau] * fQe)    ;Qe sacling
      tauC = tauC * (1.0 - fOmega)   ;tau scaling, phase func truncation
      
       temparg = reform(ratio#(reform(tauC,nwl*nRE)),nscat,nwl,nRE)
      temparg1 = temparg
      temparg = (temparg lt exptol)*temparg     
      EXP1= (temparg1 lt exptol)*EXP(-temparg*1.d0)
      
      
      temp[*,itau,*,*] = ssAlbedo[*, *] * fPhase[*,*, *] * (1 - EXP1[*,*, *]) 
    end



  ENDELSE
  
  
  ssRefl = (reform(diag_matrix(1.0/(4.0 * MUPLUSMU0))#reform(temp,nscat,ntau*nwl*nRE,/overwrite),nscat,ntau,nwl,nRE))
  
end