
; NAME :    
;           getOceanReflForGivenWindSpeed_flip
; PURPOSE :
;           Calculate reflectance over ocean surface, given the satellite geometry, wind speed 
;           from MS C6 LUTs
; CALLING :
;           liquid_refl = getOceanReflForGivenWindSpeed_flip(pf,lut,ssc,lut_wspeeds,sol, view, azm,wspeed,'LIQUID')
;           ice_refl    = getOceanReflForGivenWindSpeed_flip(pf,lut,ssc,lut_wspeeds,sol, view, azm,wspeed,'ICE')
;                        sol, view, azm can be scalars or vectors of the same length
;                        wspeed can be a scalar or a vector
; INPUTS  :
;           pf  = Structure containing phasefunction data
;           lut = Array of structures containing MS ocean reflectances for the LUT wind speeds (for C6 # of wind speeds are 3)
;           ssc = Structure with Single scattering properties and LUT RE & tau values
;           lut_wspeeds = LUT wind speeds (array)
;           sol    = Solar Angle in degrees
;           view   = View Angle in degrees
;           azm    = Azimuth Angle in degrees
;           wspeed = Wind speed at which the reflectence is needed
;           PHASE     = Liquid or Ice phase
; OUTPUT  :
;           Array containing reflectance over ocean. 
;           1st DIM = DIMS of input wspeed
;           2nd DIM = DIMS of sol, view, azm angle vector (similar dims for three angles)
; 
; EXTERNAL CALLS:     
;           calcSingScatPartOfReflectancenew  (calculate SS part) 
;           interpolateMSRefl_flip (interpolate MS reflectance)
;              
; HISTORY:
; Written: Nandana Amarasinghe, SSAI @ NASA/GSFC, 2017.
;


FUNCTION getOceanReflForGivenWindSpeed_flip,pfStruct,lutStruct,sscStruct,lut_wspeeds,solarAng,viewAng,azmAng,wspeed,phase


  nwspd = n_elements(lut_wspeeds)
  sortidx = sort(lut_wspeeds)
  sortedWSPD = lut_wspeeds[sortidx]
  nspd = n_elements(wspeed)
  

  im = value_locate(sortedWSPD,wspeed)
  
calcSingScatPartOfReflectancenew,pfStruct,sscStruct,solarAng,viewAng,azmAng,ssRefl, PHASE=phase,/OCEAN

  ;help,ssRefl
  DIMS = size(ssRefl,/DIMENSIONS)
  
   temp = fltarr([nspd,size(ssRefl,/DIMENSIONS)])
  ; help,temp
   
    idx=where(im lt 0, ct)
    if(ct gt 0)then begin
      interpolateMSRefl_flip, lutstruct[0],interpMSR
      temp[idx,*,*,*,*]=transpose(interpMSR)+ssRefl
     endif
     idx=where(im gt nwspd-1, ct)
     if(ct gt 0)then begin
         interpolateMSRefl_flip, lutstruct[nwspd-1],interpMSR
          temp[idx,*,*,*,*]=transpose(interpMSR)+ssRefl
      endif
     idx = where((im ge 0) and (im lt nwspd-1), ct)
     if(ct gt 0)then begin
      for ict = 0, ct-1 do begin
      
         interpolateMSRefl_flip, lutstruct[im[idx[ict]]], interpMSR1
         interpolateMSRefl_flip, lutstruct[im[idx[ict]]+1], interpMSR2
         temp[idx[ict],*,*,*,*] = ( transpose(interpMSR1) * (sortedWSPD[im[idx[ict]]+1]-wspeed[idx[ict]]) + $ 
                                transpose(interpMSR2) * (wspeed[idx[ict]]-sortedWSPD[im[idx[ict]]]))/ $ 
                                (sortedWSPD[im[idx[ict]]+1]-sortedWSPD[im[idx[ict]]]) + ssRefl[*,*,*,*]
       end 
     end
      
     
  RETURN, reform(temp)

END
