; NAME :    
;           readLambOrOceanLUTData
; PURPOSE :
;           Read LUT data (Land or Ocean) into IDL structures
;  
; CALLING :
;           readLandOrOceanLUTData, lutname,reflStruct,sscStruct 
;           ;;; Outputs LUT MS reflectances and relevant angles and SS properties
;           
;           readLandOrOceanLUTData, lutname,reflStruct,sscStruct,/land
;           ;;;Outputs LUT MS reflectances & fluxes and relevant angles and SS properties
;           readLandOrOceanLUTData, lutname,reflStruct,sscStruct,/ocean;;
;           ;;;Outputs LUT MS reflectances and relevant angles  and aerosol and rayleigh properties
;           
; INPUTS  :
;           lutname  = Name of the LUT (make sure to match with land and ocean correctly)
;    
; KEY WORDS:
;           /land : Reads reflectances and fluxes from the lambertian/land LUT
;           /ocean: Reads reflectances and aerosol/rayleigh data from ocean LUTs
;           
; OUTPUT  :
;           reflStruct : structure containing reflectance data (see the code to see variables) 
;           sscStruct  : single scattering properites and (aerosol + reyleigh for ocean LUT)
; 
; EXTERNAL CALLS:     
;           getDatafromHDFFile
;           
; HISTORY:
; Written: Nandana Amarasinghe, SSAI @ NASA/GSFC, 2017.
;



pro readLandOrOceanLUTData, lutname,solang, viewang, azmang,reflStruct,sscStruct,LAND=land,OCEAN=ocean


    
    

    variablesrefl=['MultiScatBDReflectance','ReflectanceSolarZenith','ReflectanceSensorZenith', $
              'ReflectanceRelativeAzimuth']
    variablesssc=['PhaseFuncNormConstant','OpticalThickness','ParticleRadius',$
               'TruncationFactor','ExtinctionCoefficient','SingleScatterAlbedo','Wavelengths']
    tagnamesrefl=['MSR','SOLZEN','SENZEN','RELAZM']
    tagnamesgeometry = ['idazm','idsol','idsen','idsenf']
    tagnamesssc=['PFNCONST','COT','RE','TRUNCF','EXTCOEFF','SSALB','WVL']
    
               

   IF KEYWORD_SET(ocean) THEN BEGIN
    variablesrlaero=['AerosolAsymParameter','AerosolSSAlbedo','AerosolOpticalThickness','RayleighOpticalThickness']
    tagnamesrlaero=['AEROG','AEROSSA','AEROCOT','RAYCOT']
    fluxStruct=0
    ;print,n_elements(variablesflux),n_elements(tagnamesflux)
  ENDIF 
 
   IF KEYWORD_SET(land) THEN BEGIN
 ; variablesflux = ['ReflectedFlux','TransmittedFlux','SphericalAlbedo','FluxSolarZenith']
    variablesflux = ['TransmittedFlux','TransmittedFlux','SphericalAlbedo','FluxSolarZenith']
  
  tagnamesrefl=['MSR','SOLZEN','SENZEN','RELAZM']
  tagnamesflux=['RFLUX','TFLUX','SPALB','FLUXSOL']
  aeroStruct=0
  ENDIF
  sd_id = HDF_SD_START(lutname, /READ)
  sds_id = hdf_sd_select(sd_id,hdf_sd_nametoindex(sd_id,variablesrefl[0]))
  HDF_SD_GETINFO,sds_id,DIMS=dimr
  hdf_sd_endaccess,sds_id
  ctvect = dimr
  ctvect[3:5]=2
  ctvectf=[ctvect[0:2],ctvect[4]]
  
   
     solarmu    = getDatafromHDFFile(sd_id,variablesrefl[1])
   viewmu     = getDatafromHDFFile(sd_id,variablesrefl[2])
   relazm     = getDatafromHDFFile(sd_id,variablesrefl[3])
  
  mu0 = cos(solang * !PI/180.0)
  mu = cos(viewang * !PI/180.0)
 
 
 nsol = n_elements(solarmu)
 nmu  =  n_elements(viewmu)
 nphi = n_elements(relazm)
 nscat = n_elements(solang)
 
  im = 0 > value_locate(solarmu,mu0) 
 iw = 0 > value_locate(viewmu, mu)  
 ip = 0 > value_locate(relazm,azmang) 
   iwf = 0 > value_locate(solarmu,mu)  ;for flux calculations 
 
 ; print,im,iw,ip
 
 
    if(im lt nsol-1)then begin
        delmu0 = solarmu(1:nsol-1) - solarmu(0:nsol-2)
        imi = 0 > (im*1.0 + (mu0 - solarmu[im])/delmu0[im]) < (nsol-1)
    endif else begin
        imi = nsol-1
    endelse
       
    if(iw lt nmu-1)then begin
      delmu = viewmu(1:nmu-1)-viewmu(0:nmu-2)
      iwi = 0 > (iw*1.0 + (mu - viewmu[iw])/delmu[iw] ) < (nmu-1)
    endif else begin
        iwi= nmu-1
    endelse
    
     if(iwf lt nsol-1)then begin
        delmu0 = solarmu(1:nsol-1) - solarmu(0:nsol-2)
        iwfi = 0 > (iwf*1.0 + (mu - solarmu[iwf])/delmu0[iwf]) < (nsol-1)
    endif else begin
        iwfi = nsol-1
    endelse
    
    
 
     if(ip lt nphi-1)then begin
      delphi = relazm(1:nphi-1) - relazm(0:nphi-2)
      ipi = 0 > (ip*1.0 + (azmang - relazm[ip])/delphi[ip]) < (nphi-1)
    endif else begin
        ipi= nphi-1
    endelse
     
 
  
  ;print,iw
  
 ; print,imi,iwi,ipi
      idvec = fltarr(4)
  idvec = [ipi, imi, iwi, iwfi]  ;needs to be before the next statement 
                                 ;which sets the start vector for the special case
  
 
  
  if(iwi ge nmu-1)then iwi = nmu-2
   if(imi ge nsol-1)then imi = nsol-2
   if(ipi ge nphi-1)then ipi= nphi-2
      if(iwfi ge nsol-1)then iwfi = nsol-2
   iidvec = floor([ipi, imi, iwi, iwfi])
   
  
  stvect = intarr(6)
  stvectf = intarr(4)
  
  stvect[3] = floor(ipi)
  stvect[4] = floor(imi)
  stvect[5] = floor(iwi)
  
  
  ;struct=create_struct(tagnames[0],data)
  reflStruct={}
  for i = 0,n_elements(variablesrefl)-1 do begin
    if(i eq 0)then data      = getDatafromHDFFile(sd_id,variablesrefl[i],CTVEC = ctvect, STARTVEC=stvect)
    if(i ne 0)then data      = getDatafromHDFFile(sd_id,variablesrefl[i])
    reflStruct=create_struct(temporary(reflStruct),tagnamesrefl[i],temporary(data)) 
  end
  for i = 0, 3 do begin
    data = idvec[i] - iidvec[i];,stvect[i+3]  
    reflStruct=create_struct(temporary(reflStruct),tagnamesgeometry[i],temporary(data)) 
  end
    
  
  sscStruct={}
  for i = 0,n_elements(variablesssc)-1 do begin
    data      = getDatafromHDFFile(sd_id,variablesssc[i])
    sscStruct=create_struct(temporary(sscStruct),tagnamesssc[i],temporary(data)) 
  end
  
  if KEYWORD_SET(land) then begin
  
      for i = 0,n_elements(variablesflux)-1 do begin
        if(i lt 2)then begin
          if(i eq 0)then stvectf[3] = iidvec[3]
          if(i eq 1)then stvectf[3] = iidvec[1]
          data = getDatafromHDFFile(sd_id,variablesflux[i],CTVEC = ctvectf, STARTVEC=stvectf)
        endif else begin
          data      = getDatafromHDFFile(sd_id,variablesflux[i])
        endelse
        reflStruct=create_struct(temporary(reflStruct),tagnamesflux[i],temporary(data)) 
      end
   end
   
  if KEYWORD_SET(ocean) then begin
      for i = 0,n_elements(variablesrlaero)-1 do begin
        data      = getDatafromHDFFile(sd_id,variablesrlaero[i])
        sscStruct=create_struct(temporary(sscStruct),tagnamesrlaero[i],temporary(data)) 
      end
   end
   
    
  
  
  HDF_SD_END, sd_id


end