#!/usr/bin/env python
""";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    This Modile contains collection of simple methods to calculate the Total reflectance (MS + SS) from C6 MODIS LUTS over
    land or ocean surface

    Required modules: numpy, phyhdf, pprint, scipy, rime. datetime


     Modified:   06 May 2017

     Programmer:

            Nandana Amarasinghe(SSAI)
            Climate and Radiation Branch
            NASA Goddard Space Flight Center
            Greenbelt, Maryland 20771, U.S.A.
            nandana.amarasinghe@ssaihq.com or nandana.d.amarasinghe@nasa.gov

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
"""

import pprint
import time                                                
import numpy as np
from scipy import interpolate
from pyhdf.SD import SD, SDC
from datetime import datetime, date, time
import timeit


   
def getPhaseFunctionData(phasename):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    Read the phase function data

     Input Parameters:   phasefile name
     Output : dictionary with phase fun related data
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """

    pfData={}
    variables=['WaterPhaseFuncVals','ScatAnglesWater','IcePhaseFuncVals','ScatAnglesIce']
    tagnames=['liqPF','liqSA','icePF','iceSA']
    file = SD(phasename, SDC.READ)
    
    for i in range(len(variables)):
        pfData[tagnames[i]] = getDatafromHDFFile(file, variables[i])
    file.end()
    return pfData

def readLambOrOceanLUTData(lutname,solang, viewang, azmang,LAND='NONE',OCEAN='NONE'):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    Read land or ocean MS reflectance data, for a particular sun-sattllite geometry (slicing)

    Input Parameters: lutname,solang, viewang, azmang,LAND='NONE',OCEAN='NONE'
     Output : dictionary with reflectance related data
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """    


    variablesrefl=['MultiScatBDReflectance','ReflectanceSensorZenith','ReflectanceSolarZenith','ReflectanceRelativeAzimuth']
    variablesssc=['PhaseFuncNormConstant','OpticalThickness','ParticleRadius','TruncationFactor','ExtinctionCoefficient','SingleScatterAlbedo','Wavelengths']
    tagnamesrefl=['MSR','SENZEN','SOLZEN','RELAZM']
    tagnamesssc=['PFNCONST','COT','RE','TRUNCF','EXTCOEFF','SSALB','WVL']

    reflData={}
    sscData = {}
    file = SD(lutname, SDC.READ)
    sds_obj = file.select(variablesrefl[0])
    ctvec = sds_obj.info()[2]
    ctvec[0:3]=[2,2,2]
    ctveclist=[[2],[2],[2],[2]]
    ctveclist[0]=ctvec
    stdvec=list(np.ones(sds_obj.info()[1],dtype=int))
    stdveclist=[[1],[1],[1],[1]]
    stdveclist[0]=stdvec
    stvec = list(np.zeros(sds_obj.info()[1],dtype=int))    
    stveclist=[[0],[0],[0],[0]]
    sds_obj.endaccess()
    mu0 = np.cos(np.deg2rad(solang))
    mu = np.cos(np.deg2rad(viewang))
    inputang=[np.cos(np.deg2rad(viewang)),np.cos(np.deg2rad(solang)), azmang]
    
    
    for i in range(3):
        sds_obj = file.select(variablesrefl[i+1])
        stvec[i]=getIndex(sds_obj.get(),inputang[i])
        stveclist[i+1]=[stvec[i]]
        sds_obj.endaccess()
        
    sds_obj = file.select(variablesrefl[2])
    solarmu=sds_obj.get()
    sds_obj.endaccess()
    ivf = getIndex(solarmu,mu)

    stveclist[0]=stvec


    
    for i in range(len(variablesrefl)):
        reflData[tagnamesrefl[i]]=getDatafromHDFFile(file, variablesrefl[i],stvec=stveclist[i],ctvec=ctveclist[i],stdvec=stdveclist[i],SECTION='TRUE')
    
    for i in range(len(variablesssc)):
        sscData[tagnamesssc[i]]=getDatafromHDFFile(file, variablesssc[i])
    if(LAND is not 'NONE'):
        stveclist[0]=stvec[2:]
        stveclist[0][0]=ivf
        stveclist[1]=stvec[2:]
        stveclist[1][0]=stvec[1]

        stveclist[3]=[ivf]
        ctveclist[1] = ctveclist[0][2:]
        ctveclist[0] = ctveclist[1]
        stdveclist[1]=stdveclist[0][2:]
        stdveclist[0]=stdveclist[1]
        variablesflux = ['ReflectedFlux','TransmittedFlux','FluxSolarZenith','FluxSolarZenith','SphericalAlbedo']
        tagnamesflux=['RFLUX','TFLUX','FLUXSOL','FLUXSEN','SPALB']
        for i in range(len(variablesflux)-1):
            reflData[tagnamesflux[i]]=getDatafromHDFFile(file, variablesflux[i],stvec=stveclist[i],ctvec=ctveclist[i],stdvec=stdveclist[i],SECTION='TRUE')
        reflData[tagnamesflux[4]]=getDatafromHDFFile(file, variablesflux[4])
    if(OCEAN is not 'NONE'):
        variablesrlaero=['AerosolAsymParameter','AerosolSSAlbedo','AerosolOpticalThickness','RayleighOpticalThickness']
        tagnamesrlaero=['AEROG','AEROSSA','AEROCOT','RAYCOT']
        for i in range(len(variablesrlaero)):
            sscData[tagnamesrlaero[i]]=getDatafromHDFFile(file, variablesrlaero[i])
    file.end()
    return reflData, sscData
    

        
        
   
def getAllPhaseFunctionValues(pFData,PFNormConst,scatAng, phase):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Interpolate phase function data

    Input Parameters:
       pFData      : phase function data (dictionary)
       FNormConst  : Normalization constants
       scatAng     : scattring angle
       phase       : Ice or Liquid   
    Output :
       array with interpolated PF data
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """

    scatAng = np.atleast_1d(scatAng)
    if (phase.upper() == 'ICE'):
        PF = pFData['icePF']
        scatAngs = pFData['iceSA']
    if (phase.upper() =='LIQUID'):
        PF = pFData['liqPF']
        scatAngs = pFData['liqSA']

 
    pfsize = len(scatAngs)
    nscat = np.size(scatAng)
    dims = PF.shape
    temp = np.zeros((dims[0],dims[1],nscat),dtype='f')
    interpPF = np.zeros((dims[0],dims[1]))
    dim01 = dims[0]*dims[1]
    
    PF = np.reshape(np.matmul(np.diagflat(1.0/PFNormConst),np.reshape(PF, (dims[0]*dims[1],pfsize))),(dims[0],dims[1],pfsize))
    iL = np.searchsorted(scatAngs,scatAng)
    idxx= np.where(iL <= 0)[0]
    lenidx = idxx.size

    if(lenidx > 0):
        temp[:,:,idxx]=np.reshape(np.repeat(PF[:,:,0],lenidx),(dims[0],dims[1],lenidx))
        
    idxx = np.where(iL >= pfsize)[0]
    lenidx = idxx.size    
    if(lenidx > 0):
     temp[:,:,idxx]=np.reshape(np.repeat(PF[:,:,pfsize-1],lenidx),(dims[0],dims[1],lenidx))
     iL[idxx]=-1
     
    idxx=np.where(iL > 0)[0]
    lenidx = idxx.size                    
    if(lenidx > 0):
     temp[:,:,idxx] = np.matmul(PF[:,:,iL[idxx]-1],np.diagflat((scatAngs[iL[idxx]]-scatAng[idxx])/(scatAngs[iL[idxx]]-scatAngs[iL[idxx]-1])))\
                                  +np.matmul(PF[:,:,iL[idxx]],np.diagflat((scatAng[idxx]-scatAngs[iL[idxx]-1])/(scatAngs[iL[idxx]]-scatAngs[iL[idxx]-1])))
     
    return temp

   
def getRLAndAeroPFValues(aerodict,scatAngle):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    Pre process aerosol, raleigh information for SS calculations


    Input Parameters:
       aerodict    : dictionary with aero/rayleigh data
       scatAng     : scattring angle
    Output :
       dictionary with aero/rayleigh data
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """
    

    scatAngle = np.atleast_1d(scatAngle)
    aerog  = aerodict['AEROG']
    aomega = aerodict['AEROSSA']
    atau   = aerodict['AEROCOT']
    RLtau  = aerodict['RAYCOT']                    

    output = {};

    nangs = scatAngle.size
    nwl = len(aerog)
    

    cosScatAng = np.cos(np.deg2rad(scatAngle))
    gsquare = aerog*aerog
  
    RLPF = 3.0 * (1 + (cosScatAng)**2)/4.0
    aeroPhase = np.zeros((nwl,nangs))
    temp = ((np.repeat(1.0+gsquare,nangs).reshape(nwl,nangs)) - np.matmul((2.0*aerog).reshape((nwl,1)),cosScatAng.reshape((1,nangs))))

    aeroPhase[:,:] = (np.repeat((1.0 - gsquare),nangs).reshape(nwl,nangs))/(np.sqrt(temp**3))
    aeroPhase[nwl-2:nwl,:] = 0.0
    output['aerophasefun'] = aeroPhase
    output['rayleighphasefun'] = RLPF
    output['aerossalb']=aomega
    output['aerotau']=atau
    output['rayleightau']=RLtau

    return output

def interpolateMSRefl(lutData, solang, viewang, azmang):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    Interpolate MS data from the LUTs


    Input Parameters:
       lutData     : MS reflectance data (dictionary)
       solang      : solar zenith angle in deg
       viewang     : sensor zenith angle in deg
       azmang      : rel azimuth angle in deg
    Output           :
       interpolated MS refl array
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """    
    solang, viewang, azmang = np.atleast_1d(solang, viewang, azmang)


    MSRefl = lutData['MSR'].transpose()
    DIMS=MSRefl.shape


    solarmu= lutData['SOLZEN']
    viewmu = lutData['SENZEN']
    viewmu = np.squeeze(viewmu)
    relazm = lutData['RELAZM']
    
    mu0 = np.cos(np.deg2rad(solang))
    mu0[np.where(mu0 < solarmu[0])]=solarmu[0]
    mu0[np.where(mu0 > solarmu[solarmu.size-1])]=solarmu[solarmu.size-1]
   
    mu = np.cos(np.deg2rad(viewang))
    mu[np.where(mu < viewmu[0])]=viewmu[0]
    mu[np.where(mu > viewmu[viewmu.size-1])]=viewmu[viewmu.size-1]

    azmang[np.where(azmang < relazm[0])]=relazm[0]
    azmang[np.where( azmang > relazm[relazm.size-1])]=relazm[relazm.size-1]

    
    nscat = mu.size
    f = interpolate.interp1d(viewmu,MSRefl)
    newarray = f(mu)
    iv = getIndex(viewmu,mu)

    newarray2 = np.zeros((DIMS[0],DIMS[1],DIMS[2],nscat),dtype='f')
    for i in range(nscat):
        f1=interpolate.interp1d(solarmu,np.squeeze(newarray[:,:,:,:,:,i]))
        f2 =interpolate.interp1d(relazm,np.squeeze(f1(mu0[i])))
        newarray2[:,:,:,i] = f2(azmang[i])
        
            
    return newarray2

   
def interpolateFluxes(solarOrViewAng, solarmu, inFlux):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    Interpolate Flux data from the LUTs


    Input Parameters:
       inFlux          : flux array
       solarOrViewAng  : angle to which flux is interpolated
       solarmu         : cosine of the angles for which flux has been computed
    Output           :
       interpolated flux array
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """    

    solarOrViewAng = np.atleast_1d(solarOrViewAng)
    mu0 = np.cos(np.deg2rad(solarOrViewAng))
    mu0[np.where(mu0 < solarmu[0])]=solarmu[0]
    mu0[np.where(mu0 > solarmu[solarmu.size-1])]=solarmu[solarmu.size-1]

    f = interpolate.interp1d(solarmu,inFlux)
    return f(mu0)
    
   
def scatAngleDeg(solarAng, viewAng, relAzm):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    Calculate scattering angle


    Input Parameters:
       solang      : solar zenith angle in deg
       viewang     : sensor zenith angle in deg
       azmang      : rel azimuth angle in deg
    Output           :
       scattering angle in degrees
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """

  
    solarAng, viewAng, relAzm = np.atleast_1d(solarAng, viewAng, relAzm)    
    solarmu = np.cos(np.deg2rad(solarAng))
    viewmu = np.cos(np.deg2rad(viewAng))

    scatAngle= -solarmu*viewmu + np.sqrt( ( 1.- solarmu**2 )*( 1.- viewmu**2 ) )*np.cos( np.deg2rad(relAzm) )
    idx= np.where(scatAngle < -1.0)
    scatAngle[idx]=-1.0
    idx= np.where(scatAngle > 1.0)
    scatAngle[idx]=1.0

    scatAngle = np.rad2deg(np.arccos(scatAngle))
  
    return scatAngle
 

   
def calcSingScatPartOfReflectancenew(pfDict, sscDict,solang, viewang, azmang, phase, OCEAN='NONE'):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    Compute Single Scattering part of the reflectance

    Input Parameters:
       pfDict      : phase function data (dict)  
       sscDict     : single scattering properties (dict)
       solang      : solar zenith angle in deg
       viewang     : sensor zenith angle in deg
       azmang      : rel azimuth angle in deg
       phase       : liquid or ice
       OCEAN       : optional, set to 'OCEAN' for over ocean
    Output           :
       array of SS reflectance for the input geometry
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """
    exptol = 12.0

    solang, viewang, azmang = np.atleast_1d(solang, viewang, azmang)
  
    scatAngle = scatAngleDeg(solang, viewang, azmang)
    UMU  = np.cos(np.deg2rad(viewang))
    UMU0 = np.cos(np.deg2rad(solang))
    MUPLUSMU0 = UMU + UMU0
    ratio = MUPLUSMU0/(UMU * UMU0)
    nscat = scatAngle.size
  
    interpPF = getAllPhaseFunctionValues(pfDict,sscDict['PFNCONST'],scatAngle, phase)
  
    tauVals = sscDict['COT']
    RE = sscDict['RE']
    dfitF = sscDict['TRUNCF']
    Qe = sscDict['EXTCOEFF']
    ssAlbedo = sscDict['SSALB']
    
    nRE = len(RE)
    ntau = len(tauVals)
    nwl = Qe.shape[1]
    fQe = np.zeros((nRE,nwl)) + 1.0
    for iwl in range(1,nwl):
        fQe[:,iwl] = Qe[:,iwl]/Qe[:,0]
  
  
    fOmega = dfitF * ssAlbedo
    fPhase = interpPF/np.reshape(np.repeat((1.0 - fOmega),nscat),(nRE,nwl,nscat)) 
  
    temp = np.zeros((nRE,nwl,ntau,nscat))
    nREfromPhase = fPhase.shape[0]

    if(nRE != nREfromPhase):
        raise NameError('Please check the to see LUT matches with the correct phase called (ICE or LIQUID)')

    if(OCEAN is 'NONE'):
       for itau in range(ntau):
          tauC = (tauVals[itau] * fQe)    #Qe sacling
          tauC = tauC * (1.0 - fOmega)   #tau scaling, phase func truncation
          tempexp = np.matmul(np.reshape(tauC,(nRE*nwl,1)), np.reshape(ratio,(1,nscat))).reshape((nRE,nwl,nscat)) #tau *(1/mu + 1/mu0)
          exptempexp = np.where(tempexp > exptol,1.0,(1.0 - np.exp(-tempexp)))
          sstemp = np.matmul(np.reshape(ssAlbedo,(nRE*nwl,1)), np.reshape(0.25/MUPLUSMU0,(1,nscat))).reshape((nRE,nwl,nscat))
          temp[:,:,itau,:] = (sstemp * fPhase * exptempexp)
                              # omega * P * {1-exp[- tau *(1/mu + 1/mu0)]}
       ssRefl = temp

    elif(OCEAN is 'OCEAN' or OCEAN is 'ocean'):
        tagnamesrlaero=['AEROG','AEROSSA','AEROCOT','RAYCOT']
        aeroDict = {key:sscDict[key] for key in tagnamesrlaero}
        aeroinfo=getRLAndAeroPFValues(aeroDict,scatAngle)
        
        temp1 = np.zeros((nRE,nwl,ntau,nscat))
        for itau in range(ntau):
            tauLR =  tauVals[itau] * fQe *(1.0 - fOmega)
            EXP0 = 1.0
            tempexp = np.matmul(np.reshape(tauLR,(nRE*nwl,1)), np.reshape(ratio,(1,nscat))).reshape((nRE,nwl,nscat))
            EXP1= np.where(tempexp > exptol,0.0,np.exp(-tempexp))
            sstemp = np.reshape(np.matmul(np.diagflat(ssAlbedo),np.reshape(fPhase, (nRE*nwl,nscat))),(nRE,nwl,nscat))
            temp1[:,:,itau,:] = temp1[:,:,itau,:] + sstemp * (EXP0 - EXP1)
            
            EXP0 = +EXP1
            for iwl in range(nwl-2):
                tauLR[:,iwl] =  tauLR[:,iwl] + aeroinfo['rayleightau'][iwl]
                tempexp = np.matmul(np.reshape(tauLR[:,iwl],(nRE,1)), np.reshape(ratio,(1,nscat))).reshape((nRE,nscat))
                
                EXP1[:,iwl,:] = np.where(tempexp > exptol,0.0,np.exp(-tempexp))
                temp1[:,iwl,itau,:] = temp1[:,iwl,itau,:] + aeroinfo['rayleighphasefun'] * 1.0 * (EXP0[:,iwl,:] - EXP1[:,iwl,:])
                EXP0[:,iwl,:] = +EXP1[:,iwl,:]
                tempOmega = ((aeroinfo['aerotau'][iwl] -0.1) * 1.0 + 0.1 * aeroinfo['aerossalb'][iwl])/aeroinfo['aerotau'][iwl]
                tauLR[:,iwl] = tauLR[:,iwl]  + aeroinfo['aerotau'][iwl]
                tempPhase = ((aeroinfo['aerotau'][iwl] -0.1) * aeroinfo['rayleighphasefun'] * 1.0 + \
                             0.1 * aeroinfo['aerossalb'][iwl] * aeroinfo['aerophasefun'][iwl])/\
                             ( (aeroinfo['aerotau'][iwl] -0.1)*1.0 + 0.1 * aeroinfo['aerossalb'][iwl] )
                tempexp = np.matmul(np.reshape(tauLR[:,iwl],(nRE,1)), np.reshape(ratio,(1,nscat))).reshape((nRE,nscat))
                EXP1[:,iwl,:] = np.where(tempexp > exptol,0.0,np.exp(-tempexp))
                temp1[:,iwl,itau,:] = temp1[:,iwl,itau,:] +  tempOmega * np.matmul((EXP0[:,iwl,:] - EXP1[:,iwl,:]),np.diagflat(tempPhase))
        ssRefl =(np.matmul(temp1, np.diagflat(0.25/MUPLUSMU0)))

    return ssRefl


   
def getReflForGivenSurfAlbedo(pfDict,lutDict,sscDict,solarAng,viewAng,azmAng,surfAlb,phase):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    Compute reflectance over land

    Input Parameters:
       pfDict      : phase function data (dict)
       lutDict     : lut data (dict)
       sscDict     : single scattering properties (dict)
       solang      : solar zenith angle in deg
       viewang     : sensor zenith angle in deg
       azmang      : rel azimuth angle in deg
       surfAlb     : surface albedo
       phase       : liquid or ice
       OCEAN       : optional, set to 'OCEAN' for over ocean
    Output           :
       array of totla reflectance for the input geometry
    Routines Called:
     calcSingScatPartOfReflectancenew
     interpolateMSRefl
     interpolateFluxes
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """    

    solarAng,viewAng,azmAng,surfAlb = np.atleast_1d(solarAng,viewAng,azmAng,surfAlb)

    ssRefl = calcSingScatPartOfReflectancenew(pfDict,sscDict,solarAng,viewAng,azmAng,phase)
    interpMSR = interpolateMSRefl(lutDict,solarAng,viewAng,azmAng)
    

    reflAsurf = (ssRefl + interpMSR)
    reflFlux = lutDict['RFLUX']
    reflFlux =    reflFlux.transpose()
    transFlux = lutDict['TFLUX']
    transFlux =    transFlux.transpose()    
    spAlbedo = lutDict['SPALB']
    solarMuRFTF = lutDict['FLUXSOL']
    outTFluxArrayMu0=interpolateFluxes(solarAng,solarMuRFTF, transFlux)
    solarMuRFTF = lutDict['FLUXSEN']
    outTFluxArrayMU =  interpolateFluxes(viewAng,solarMuRFTF, reflFlux)
    del reflFlux,transFlux 
    nalb = surfAlb.size
    nang =    solarAng.size
    DIMS = reflAsurf.shape
    temp = np.zeros(((reflAsurf.shape)+(nalb,)))
    for i in range(nalb):
        temp[:,:,:,:,i] = reflAsurf[:,:,:,:] + np.reshape(np.matmul(np.diagflat(surfAlb[i]/( 1.0 - surfAlb[i] * spAlbedo)),\
                                                np.reshape((outTFluxArrayMu0 * outTFluxArrayMU),(DIMS[0]*DIMS[1]*DIMS[2],nang))),(DIMS[0],DIMS[1],DIMS[2],nang))
 
    return temp

   
def getOceanReflForGivenWindSpeed(pfDict,lutDict,sscDict,lut_wspeeds,solarAng,viewAng,azmAng,wspeed,phase):
    """;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    Compute reflectance over ocean

    Input Parameters:
       pfDict      : phase function data (dict)
       lutDict     : lut data (dict)
       sscDict     : single scattering properties (dict)
       lut_wspeeds : LUT wind speeds (currently 3, 7, 15)
       solang      : solar zenith angle in deg
       viewang     : sensor zenith angle in deg
       azmang      : rel azimuth angle in deg
       wspeed      : wind speed (m/s)
       phase       : liquid or ice
    Output           :
       array of totla reflectance for the input geometry
    Routines Called:
     calcSingScatPartOfReflectancenew
     interpolateMSRefl
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    """    

    solarAng,viewAng,azmAng,wspeed = np.atleast_1d(solarAng,viewAng,azmAng,wspeed)    

    nwspd = len(lut_wspeeds)
    nspeed= wspeed.size
    sortidx = np.argsort(lut_wspeeds)
    sortedWSPD = np.array(lut_wspeeds)[sortidx]

    ssRefl = calcSingScatPartOfReflectancenew(pfDict,sscDict,solarAng,viewAng,azmAng,phase,OCEAN='OCEAN')

    temp = np.zeros((ssRefl.shape+(nspeed,)))
    for i in range(nspeed):
        iL = np.searchsorted(sortedWSPD,wspeed[i])
    
        if (iL== 0 and wspeed[i] <= sortedWSPD[0]):
            interpMSR = interpolateMSRefl(lutDict[0],solarAng,viewAng,azmAng)
            temp[:,:,:,:,i] = interpMSR+ssRefl
        elif(iL >= nwspd and wspeed[i] >= sortedWSPD[nwspd-1]):
            interpMSR = interpolateMSRefl(lutDict[nwspd-1], solarAng,viewAng,azmAng)
            temp[:,:,:,:,i] = interpMSR+ssRefl    
        else:
            im=iL-1
            interpMSR1 = interpolateMSRefl(lutDict[im], solarAng,viewAng,azmAng)
            interpMSR2 = interpolateMSRefl(lutDict[im+1], solarAng,viewAng,azmAng)
            temp[:,:,:,:,i] = ( interpMSR1 * (sortedWSPD[im+1]-wspeed[i]) + \
              interpMSR2 * (wspeed[i]-sortedWSPD[im]))/(sortedWSPD[im+1]-sortedWSPD[im])
            del interpMSR1, interpMSR2
            temp[:,:,:,:,i]=temp[:,:,:,:,i] + ssRefl

    return temp




   
def getDatafromHDFFile(fileID, variable,stvec=[],ctvec=[],stdvec=[],SECTION='NONE'):
    
    if(SECTION is not 'NONE'):
       sds_obj = fileID.select(variable)
       new_stvec = []
       new_stdvec=[]
       for item in stvec:
           new_stvec.append(int(item))
       for item in stdvec:
           new_stdvec.append(int(item))
       return sds_obj.get(start=new_stvec,count=ctvec, stride=new_stdvec)
    else:
        sds_obj = fileID.select(variable)
        return sds_obj.get()

def getIndex(array1d,value):
    idx = np.searchsorted(array1d,value)
    if(idx <= 0):idx = 1
    if(idx >= array1d.size):idx = idx-1
    return idx-1



def createHDFFile(outname,reflWater, reflIce,angs,ag, rew,rei,tau, wvl,LAND='NONE',OCEAN='NONE'):

    dt=datetime.now()

    solar=angs[0]
    view=angs[1]
    azm=angs[2]
    scaleFactor = 1.0
    FILE_ID = SD(outname ,SDC.WRITE|SDC.CREATE)
    if(LAND is not 'NONE'):
        FILE_ID.TITLE ='TOA REFLECTANCE OVER LAND'
        FILE_ID.SURFACE_ALBEDO = ag


    if(OCEAN is not 'NONE'):
        FILE_ID.TITLE='TOA REFLECTANCE OVER OCEAN'
        FILE_ID.WIND_SPEED =ag
        FILE_ID.WIND_SPEED_UNITS='m/s'


    att = FILE_ID.attr('SOLARANG')
    att.set(SDC.FLOAT32, solar)
    att = FILE_ID.attr('VIEWANG')
    att.set(SDC.FLOAT32, view)

    att = FILE_ID.attr('AZMANG')
    att.set(SDC.FLOAT32, azm)
    FILE_ID.CREATION_DATE=dt.strftime("%A, %d. %B %Y %I:%M%p")

    s = reflWater.shape
    d1=FILE_ID.create('reflectanceWater', SDC.FLOAT64, (s[0],s[1],s[2]))
    if(LAND is not 'NONE'):
        d1.Description ='Lambertian/land Water C6 Reflectance'
    if(OCEAN is not 'NONE'):
        d1.Description ='Ocean Water C6 Reflectance'
        
    
    dim1 = d1.dim(0)
    dim2 = d1.dim(1)
    dim3 = d1.dim(2)
    dim3.setname('NumOfCOT')
    dim2.setname('NumOfWvl')
    dim1.setname('NumOfRE_Water')
    reflWater[:,s[1]-1,:]=-9999.0
    d1.set(reflWater)
    d1.Scale_Factor = scaleFactor
    d1.endaccess()


    d1 = FILE_ID.create('ParticleRadiusWater',SDC.FLOAT32, (s[0]))
    d1.set(rew)
    d1.Scale_Factor = scaleFactor
    d1.Description= 'Effective Radii for water'
    d1.Units = 'microns'
    d1.endaccess()


    s = reflIce.shape
    d1=FILE_ID.create('reflectanceIce', SDC.FLOAT64, (s[0],s[1],s[2]))
    if(LAND is not 'NONE'):
        d1.Description ='Lambertian/land Ice C6 Reflectance'
    if(OCEAN is not 'NONE'):
        d1.Description ='Ocean Ice C6 Reflectance'
        
    
    dim1 = d1.dim(0)
    dim2 = d1.dim(1)
    dim3 = d1.dim(2)
    dim3.setname('NumOfCOT')
    dim2.setname('NumOfWvl')
    dim1.setname('NumOfRE_Ice')
    reflIce[:,s[1]-1,:]=-9999.0
    d1.set(reflIce)
    d1.Scale_Factor = scaleFactor
    d1.endaccess()


    d1 = FILE_ID.create('ParticleRadiusIce',SDC.FLOAT32, (s[0]))
    d1.set(rei)
    d1.Scale_Factor = scaleFactor
    d1.Description= 'Effective Radii for ice'
    d1.Units = 'microns'
    d1.endaccess()

    d1=FILE_ID.create('WaveLengths', SDC.FLOAT32, (s[1]))
    d1.set(wvl)
    d1.Scale_Factor = scaleFactor
    d1.Description = 'Wavelengths LUT'
    d1.Units = 'microns'
    d1.endaccess()

    d1 = FILE_ID.create('OpticalThickness', SDC.FLOAT32, (s[2]))
    d1.set(tau)
    d1.Scale_Factor = scaleFactor
    d1.Description = 'COT LUT'
    d1.Units = 'none'
    d1.endaccess()

    FILE_ID.end()

    return 0



